@@ -25,7 +25,7 @@
 
 #include <klocale.h>
 
-SKGTrackerObject::SKGTrackerObject(const SKGDocument* iDocument, int iID): SKGNamedObject(iDocument, "v_refund", iID)
+SKGTrackerObject::SKGTrackerObject(SKGDocument* iDocument, int iID): SKGNamedObject(iDocument, "v_refund", iID)
 {
 }
 
@@ -34,81 +34,88 @@ SKGTrackerObject::~SKGTrackerObject()
 }
 
 SKGTrackerObject::SKGTrackerObject(const SKGTrackerObject& iObject)
-                :SKGNamedObject(iObject)
+        :SKGNamedObject(iObject)
 {
 }
 
 SKGTrackerObject::SKGTrackerObject ( const SKGObjectBase& iObject )
 
 {
-        if ( iObject.getRealTable() =="refund" ) {
-                copyFrom ( iObject );
-        } else {
-                *this=SKGNamedObject::SKGNamedObject ( iObject.getDocument(), "v_refund", iObject.getID() );
-        }
+    if ( iObject.getRealTable() =="refund" ) {
+        copyFrom ( iObject );
+    } else {
+        *this=SKGNamedObject ( iObject.getDocument(), "v_refund", iObject.getID() );
+    }
 }
 
 const SKGTrackerObject& SKGTrackerObject::operator= (const SKGObjectBase& iObject)
 {
-        copyFrom(iObject);
-        return *this;
+    copyFrom(iObject);
+    return *this;
 }
 
-SKGError SKGTrackerObject::createTracker ( const SKGDocumentBank* iDocument,
-                const QString& iName,
-                SKGTrackerObject& oTracker,
-                bool iSendPopupMessageOnCreation )
+SKGError SKGTrackerObject::createTracker ( SKGDocumentBank* iDocument,
+        const QString& iName,
+        SKGTrackerObject& oTracker,
+        bool iSendPopupMessageOnCreation )
 {
-        SKGError err;
-        SKGTRACEINRC ( 10, "SKGTrackerObject::createTracker", err );
+    SKGError err;
+    SKGTRACEINRC ( 10, "SKGTrackerObject::createTracker", err );
 
-        //Check if refund is already existing
+    //Check if refund is already existing
+    if (iName.isEmpty())
+    {
+        oTracker=SKGTrackerObject(NULL, 0);
+    }
+    else
+    {
         SKGObjectBase::getObject ( iDocument, "v_refund", "t_name='"+SKGServices::stringToSqlString ( iName ) +'\'', oTracker );
         if ( oTracker.getID() ==0 ) {
-                //No, we have to create it
-                oTracker = SKGTrackerObject ( ( SKGDocumentBank* ) iDocument );
-                err=oTracker.setName ( iName );
-                if ( err.isSucceeded() ) err=oTracker.save();
+            //No, we have to create it
+            oTracker = SKGTrackerObject ( iDocument );
+            err=oTracker.setName ( iName );
+            if ( err.isSucceeded() ) err=oTracker.save();
 
-                if ( err.isSucceeded() && iSendPopupMessageOnCreation && iDocument ) {
-                        ( ( SKGDocumentBank* ) iDocument )->sendMessage (  i18n( "Tracker [%1] has been created" , iName ), true );
-                }
+            if ( err.isSucceeded() && iSendPopupMessageOnCreation && iDocument ) {
+                err=iDocument->sendMessage (  i18nc( "Information message", "Tracker '%1' has been created" , iName ), true );
+            }
         }
+    }
 
-        return err;
+    return err;
 }
 
 SKGError SKGTrackerObject::getSubOperations(SKGListSKGObjectBase& oSubOperations) const
 {
-        SKGError err = getObjects(getDocument(), "v_suboperation",
-                                  "r_refund_id="+ SKGServices::intToString(getID()) ,
-                                  oSubOperations);
-        return err;
+    SKGError err = getObjects(getDocument(), "v_suboperation",
+                              "r_refund_id="+ SKGServices::intToString(getID()) ,
+                              oSubOperations);
+    return err;
 }
 
 SKGError SKGTrackerObject::setClosed(bool iClosed)
 {
-        return setAttribute("t_close", iClosed ? "Y" :"N");
+    return setAttribute("t_close", iClosed ? "Y" :"N");
 }
 
 bool SKGTrackerObject::isClosed() const
 {
-        return (getAttribute("t_close")=="Y" ? true : false);
+    return (getAttribute("t_close")=="Y" ? true : false);
 }
 
 SKGError SKGTrackerObject::setComment(const QString& iComment)
 {
-        return setAttribute("t_comment", iComment);
+    return setAttribute("t_comment", iComment);
 }
 
 QString SKGTrackerObject::getComment() const
 {
-        return getAttribute("t_comment");
+    return getAttribute("t_comment");
 }
 
 double SKGTrackerObject::getCurrentAmount() const
 {
-        return SKGServices::stringToDouble(getAttribute("f_CURRENTAMOUNT"));
+    return SKGServices::stringToDouble(getAttribute("f_CURRENTAMOUNT"));
 }
 
 #include "skgtrackerobject.moc"
