@@ -162,6 +162,12 @@ public slots:
      */
     virtual void onSwitchAntialiasing();
 
+signals:
+    /**
+     * View is resized
+     */
+    void resized();
+
 protected:
     /**
      * Event filtering
@@ -169,7 +175,7 @@ protected:
      * @param event event
      * @return In your reimplementation of this function, if you want to filter the event out, i.e. stop it being handled further, return true; otherwise return false.
      */
-    bool eventFilter ( QObject *object, QEvent *event );
+    virtual bool eventFilter ( QObject *object, QEvent *event );
 
 private slots:
     void showMenu ( const QPoint& pos );
